import subprocess
import argparse
import sys
"""
Contains help method to start,stop and restart the server.
Called simply by calling this script with the flag --mode.
The flag is optional. If no flag is present it will start the server.
If the flag is present it will do one of four things:
1. --mode START, will start the server
2. --mode STOP, will stop the server
3. --mode RESTART, will restart the server
4. --mode XXX, will give warning due to unknown mode
"""


def start_server():
    """
    Starts the server. Note that this will not finish until the server is terminated
    so the command prompt will be occupied for the entirety of the servers life.
    """
    print("Starting build")
    subprocess.run("docker-compose build --no-cache", shell=True)
    print("Finish build, starting run")
    subprocess.run("docker-compose up -V", shell=True)
    print("Server shutdown")


def restart_server():
    """
    Restarts the server. Note that this will not finish until the new server is terminated
    so the command prompt will be occupied for the entirety of the new servers life.
    """
    print("Stopping current run")
    subprocess.run("docker-compose stop", shell=True)
    print("Containers stopped, teardown initiated")
    subprocess.run("docker-compose down", shell=True)
    print("Teardown complete")
    print("Restarting containers")
    subprocess.run("docker-compose build --no-cache", shell=True)
    print("Finish build, starting run")
    subprocess.run("docker-compose up -V", shell=True)
    print("Server shutdown")


def stop_server():
    """
    Stops the server. This will terminate the server and free the command prompt that
    started the server(whether it was a restart or not). The moment its done it will exit.
    """
    print("Stopping containers")
    subprocess.run("docker-compose stop", shell=True)
    print("Containers stopped, teardown initiated")
    subprocess.run("docker-compose down", shell=True)
    print("Teardown complete")


def main(arg_list):
    if arg_list.mode == "START":
        start_server()
    elif arg_list.mode == "STOP":
        stop_server()
    elif arg_list.mode == "RESTART":
        restart_server()
    else:
        print("NO SUCH MODE, exiting with -1.")
        sys.exit(-1)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Help functions to hide multiple docker commands")
    parser.add_argument('--mode', dest="mode", type=str, choices={"START", "STOP", "RESTART"}, default='START',
                        help="The different modes you can call")
    args = parser.parse_args()
    main(args)
